
/*
he5mu.c

This program read metadata from HDF5 file and write metadata 
back to the HDF5 file.

Zhangshi Yin
 
2003.3.31
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ctype.h>
#include "hdf5.h"
#include "odldef.h"

/* Read metadata from HDF5. */
int MetaRead(const char *MetadataName);

/* Write metadata to HDF5. */ 
int MetaWrite(hid_t fid, char write, char update);

/* Write metadata group. */
int WriteGroup(FILE *MFile, char *MetadataName, char *GroupName);

/* Write metadata dataset. */
int WriteDataset(char *MetadataName, char *MetaBuffer, size_t TotalSize);

/* Inquiry function and its pamameter passed from H5Giterate. */
herr_t InquireMetadata(hid_t loc_id, const char *MetadataName, void *opdata);

/* Read metadata to MetadataInHDF. */
herr_t ReadMetadataInHDF(hid_t loc_id, const char *MetadataName, void *opdata);

/* Unlink metadata. */
herr_t Unlink(hid_t loc_id, const char *MetadataName, void *opdata);

/* Get a value of ODL attribute. */
int GetValue(FILE *metadataFile, char *GroupName, char *attributeName, char *valueName, char *returnValue);

/* Check INVENTORYMETADATA, COREMETADATA, COLLECTIONMETADATA groups */
int MetaGroup(FILE *metadataFile,  char *GroupName);

void CheckValue(FILE *TempFile, FILE *MetadataFile, char *GroupName);

/* Print help message. */
void PrintHelp(void);

/* Convert low case to upcase. */ 
void LtoU (char *inLine, char *outLine);

/* Convert space line to no space line. */
void noSpaceLine(char *inLine, char *outLine);

FILE *MetadataFile, *MetadataInHDF, *TempFile;
hid_t gid;
int Group[3];
char CorU;

int main(int argc, char *argv[])
{
  hid_t fid;
  int  i, j, FlagLength, DataFile=0, MetaFile=0;
  char HDFEOS5FileName[256];
  char MetadataFileName[256];
  AGGREGATE root;
  int status = 0;
  char aLine[1024];
  int HDFGroup, METGroup;
  char read='n', write='n', update='n';

  /*
  Get comand-line argument.
  */
  if(argc==1)
  {
    PrintHelp();
  }
  for (i=1; i<argc; i++)
  {
    if (argv[i][0]=='-')
    {
      FlagLength=strlen(argv[i]);
      for (j=1; j<FlagLength; j++)
      {
        switch(argv[i][j])
        {
          case 'h':
              PrintHelp();
          case 'x':
              read='y';
	      if(i < (argc - 1)) {
		strcpy(MetadataFileName, argv[i+1]);
		fprintf(stderr,"Metadata file: %s.\n", MetadataFileName);
		MetaFile=1;
		i++;
	      }
              break;
          case 'f':
              write='y';
	      if(i < (argc - 1)) {
		strcpy(MetadataFileName, argv[i+1]);
		fprintf(stderr,"Metadata file: %s.\n", MetadataFileName);
		MetaFile=1;
		i++;
	      }
              break;
          case 'm':
              update='y';
	      if(i < (argc - 1)) {
		strcpy(MetadataFileName, argv[i+1]);
		fprintf(stderr,"Metadata file: %s.\n", MetadataFileName);
		MetaFile=1;
		i++;
	      }
              break;
	default:
	  fprintf(stderr,"Unknown option %s\n", argv[i]);
	  exit(1);
        }
      }
    }
    else {
      DataFile = 1;
      strcpy(HDFEOS5FileName, argv[i]);
      fprintf(stderr, "HDFEOS5 file: %s.\n", HDFEOS5FileName);
    }
  }

  if (DataFile==0)
  {
    fprintf(stderr, "No HDFEOS5 file.\n");
    PrintHelp();
  }
  if(MetaFile == 0)
    sprintf(MetadataFileName, "standard %s", (read ? "output" : "input"));

  if (read=='n' && write=='n' && update=='n')
  {
    fprintf(stderr, "Must specify one of -x|-u|-m.\n");
    PrintHelp();
  }

  /* 
  Read metadata from HDF5 file.  
  */
  if (read=='y')
  {
    /* Open HDFEOS5 file. */
    if ((fid=H5Fopen(HDFEOS5FileName, H5F_ACC_RDONLY, H5P_DEFAULT))<0)
    {
      fprintf(stderr, "Cannot open HDFEOS5 file: %s.\n", HDFEOS5FileName);
      exit(1);
    }

    /* Open metadata file. */
    if(MetaFile == 1)
      MetadataFile=fopen(MetadataFileName, "w+");
    else
      MetadataFile = stdout;

    if (MetadataFile==NULL)
    {
      fprintf(stderr, "Cannot open metadata file %s.\n", MetadataFileName);
      exit(1);
    }

    fprintf(stderr,"It is dumping metadata ...\n"); 

    /*
    Iterate through the file.
    */
    TempFile=tmpfile(); 
    if ((H5Giterate(fid, "/HDFEOS INFORMATION", NULL, InquireMetadata, NULL))<0)
    {
      fprintf(stderr, "Cannot iterate HDFEOS5 file.\n");
      exit(1);
    }

    fseek(TempFile, 0, 0);
    while (fgets(aLine, 1024, TempFile))
    {
      if (strcmp(aLine, "END\n")!=0 && strcmp(aLine, "END")!=0 && strcmp(aLine, "!")!=0)
      {
        fprintf(MetadataFile, "%s", aLine); 
      }
    }
    fprintf(MetadataFile, "END\n\n"); 
    fclose(TempFile);

    if (fclose(MetadataFile) == -1)
    {
      fprintf(stderr,"Cannot close metadata file %s.\n", MetadataFileName);
      exit(1);
    }
    if (H5Fclose(fid) == -1)
    {
      fprintf(stderr,"Cannot close HDFEOS file %s.\n", HDFEOS5FileName);
      exit(1);
    }

    fprintf(stderr,"Finished metadata dumping.\n");
  }

  /*
  Write metadata to HDF5 file.
  */
  if (write=='y' || update=='y')
  {
    /* Open HDFEOS5 file. */
    if ((fid=H5Fopen(HDFEOS5FileName, H5F_ACC_RDWR, H5P_DEFAULT))<0)
    {
      fprintf(stderr, "Cannot open HDFEOS5 file: %s.\n", HDFEOS5FileName);
      exit(1);
    }

    /* Open metadata file. */
    if(MetaFile == 1)
      MetadataFile=fopen(MetadataFileName, "r");
    else {
      /* metadata file is stdin, so read it into a temporary file
	 (that we can rewind and read again) */
      MetadataFile = tmpfile();
      while ((i= fread(aLine, 1, 1024, stdin)) > 0)
	fwrite(aLine, 1, 1024, MetadataFile);
      fseek(MetadataFile, 0, 0);
    }

    if (MetadataFile==NULL)
    {
      fprintf(stderr, "Cannot open metadata file %s.\n", MetadataFileName);
      exit(1);
    }

    /*
    Check syntax of MetadataFile.
    */
    root=NewAggregate(NULL, KA_GROUP, "test", NULL);
    status=ReadLabel(MetadataFile, root);
    if (status!=1)
    {
      fprintf(stderr, "There are syntax errors in input metadata file.\n");
      exit(1);
    }
    RemoveAggregate(root);

    /*
    Read all metadata to MetadataInHDF.
    */
    MetadataInHDF=tmpfile();
    if ((H5Giterate(fid, "/HDFEOS INFORMATION", NULL, ReadMetadataInHDF, NULL))<0)
    {
      fprintf(stderr, "Cannot iterate HDFEOS5 file.\n");
      exit(1);
    }

    /* Process MetadataInHDF to TempFile. */
    fseek(MetadataInHDF, 0, 0);
    TempFile=tmpfile();
    while (fgets(aLine, 1024, MetadataInHDF))
    {
      if (strcmp(aLine, "END\n")!=0 && strcmp(aLine, "END")!=0 && strcmp(aLine, "!")!=0)
      {
        fprintf(TempFile, "%s", aLine); 
      }
    }
    fprintf(TempFile, "END\n\n"); 
    
    fclose(MetadataInHDF);
   
    /* Check COLLECTION, INVENTORY, ARCHIVE groups. */
    Group[0]=0;
    Group[1]=0;
    Group[2]=0;
    if (update=='y')
    {
      CorU='m';
      Group[0]=MetaGroup(MetadataFile, "COLLECTIONMETADATA");
      Group[1]=MetaGroup(MetadataFile, "INVENTORYMETADATA");
      Group[2]=MetaGroup(TempFile, "ARCHIVEMETADATA");
    }

    /* Check INVENTORY values. */
    HDFGroup=MetaGroup(TempFile, "INVENTORYMETADATA");
    METGroup=MetaGroup(MetadataFile, "INVENTORYMETADATA");
    if ((HDFGroup==1) && (METGroup==1))
    {
      CheckValue(TempFile, MetadataFile, "INVENTORYMETADATA");
    }

    MetaWrite(fid, write, update);

    fclose(TempFile);

    if (fclose(MetadataFile) == -1)
    {
      fprintf(stderr,"Cannot close metadata file %s.\n", MetadataFileName);
      exit(1);
    }
    if (H5Fclose(fid) == -1)
    {
      fprintf(stderr,"Cannot close HDFEOS file %s.\n", HDFEOS5FileName);
      exit(1);
    }

    fprintf(stderr,"Finished metadata writing.\n"); 
  }
  return 0;
}

/*
Inquiry function and its pamameter passed from H5Giterate.
*/
herr_t InquireMetadata(hid_t loc_id, const char *MetadataName, void *opdata)
{
  H5G_stat_t statbuf;

  H5Gget_objinfo(loc_id, MetadataName, 0, &statbuf);

  switch (statbuf.type)
  {
    case H5G_DATASET:
         if (strncmp(MetadataName, "Struct", 6)==0) return 0;
         fprintf(stderr,"Dumping metadata: %s.\n", MetadataName);
         gid=loc_id;
         MetaRead(MetadataName);
         break;
  }
  return 0;
}

/* Read metadata to MetadataInHDF. */
herr_t ReadMetadataInHDF(hid_t loc_id, const char *MetadataName, void *opdata)
{
  H5G_stat_t statbuf;

  H5Gget_objinfo(loc_id, MetadataName, 0, &statbuf);

  switch (statbuf.type)
  {
    case H5G_DATASET:
         if (strncmp(MetadataName, "Struct", 6)==0) return 0;
         gid=loc_id;
         TempRead(MetadataName);
         break;
  }
  return 0;
}

/*
Inquiry function and its pamameter passed from H5Giterate.
*/
herr_t Unlink(hid_t loc_id, const char *MetadataName, void *opdata)
{
  H5G_stat_t statbuf;

  H5Gget_objinfo(loc_id, MetadataName, 0, &statbuf);

  switch (statbuf.type)
  {
    case H5G_DATASET:
         if (CorU=='m')
         {
         if (Group[0]==1)
         {
         if (strncmp(MetadataName, "Coll", 4)==0 ||
             strncmp(MetadataName, "coll", 4)==0 ||
             strncmp(MetadataName, "COLL", 4)==0 
            )
         {
           if (Group[0]==1)
           {
             if ((H5Gunlink(loc_id, MetadataName)) < 0) 
             {
               fprintf(stderr,"Cannot unlink old metadata %s.\n", MetadataName);
               exit(1);
             }
           }
         }
         }
         if (Group[1]==1)
         {
         if (strncmp(MetadataName, "Core", 4)==0 || 
             strncmp(MetadataName, "core", 4)==0 ||
             strncmp(MetadataName, "CORE", 4)==0 
            )
         {
           if (Group[1]==1)
           {
             if ((H5Gunlink(loc_id, MetadataName)) < 0) 
             {
               fprintf(stderr,"Cannot unlink old metadata %s.\n", MetadataName);
               exit(1);
             }
           }
         }
         }
         if (Group[2]==1)
         {
         if (strncmp(MetadataName, "Arch", 4)==0 ||
             strncmp(MetadataName, "arch", 4)==0 ||
             strncmp(MetadataName, "ARCH", 4)==0 
            )
         {
           if (Group[2]==1)
           {
             if ((H5Gunlink(loc_id, MetadataName)) < 0) 
             {
               fprintf(stderr,"Cannot unlink old metadata %s.\n", MetadataName);
               exit(1);
             }
           }
         }
         } 
         }
         else
         {
         if (strncmp(MetadataName, "Coll", 4)==0 ||
             strncmp(MetadataName, "coll", 4)==0 ||
             strncmp(MetadataName, "COLL", 4)==0 ||
             strncmp(MetadataName, "Core", 4)==0 || 
             strncmp(MetadataName, "core", 4)==0 ||
             strncmp(MetadataName, "CORE", 4)==0 ||
             strncmp(MetadataName, "Arch", 4)==0 ||
             strncmp(MetadataName, "arch", 4)==0 ||
             strncmp(MetadataName, "ARCH", 4)==0 
            )
         {
           if ((H5Gunlink(loc_id, MetadataName)) < 0) 
           {
             fprintf(stderr,"Cannot unlink old metadata %s.\n", MetadataName);
             exit(1);
           }
         }
         }
         break;
  }
  return 0;
}

/*
Read HDF5 metadata.
*/
int MetaRead(const char *MetadataName)
{
  char *MetaBuffer;
  hid_t did;
  hid_t dtype;
  hid_t dspace;
  size_t TotalSize;

  /* Open dataset */
  if ((did=H5Dopen(gid, MetadataName)) < 0) 
  {
    fprintf(stderr,"Cannot open dataset %s.\n", MetadataName);
    exit(1);
  }

  /* Get dataset type id */
  if ((dtype=H5Dget_type(did)) < 0) 
  {
    fprintf(stderr,"Cannot get dataset type %s.\n", MetadataName);
    exit(1);
  }  

  /* Get dataspace id */ 
  if ((dspace=H5Dget_space(did)) < 0) 
  {
    fprintf(stderr,"Cannot get dataset space %s.\n", MetadataName);
    exit(1);
  }   
 
  TotalSize=H5Tget_size(dtype);
  MetaBuffer=(char *)calloc(TotalSize+100, 1);

  /* Read dataset */ 
  if (H5Dread(did, dtype, dspace, H5S_ALL,H5P_DEFAULT, MetaBuffer) < 0 ) 
  {
    fprintf(stderr,"Cannot read dataset %s.\n", MetadataName);
    exit(1);
  }

  fprintf(TempFile, "%s", MetaBuffer);

  /* Close dataspace */ 
  if (H5Sclose(dspace) == -1)
  {
    fprintf(stderr,"Cannot close dataspace for %s.\n", MetadataName);
    exit(1);
  }
  
  /* Close datatype */ 
  if (H5Tclose(dtype) == -1)
  {
    fprintf(stderr,"Cannot close type for %s.\n", MetadataName);
    exit(1);
  }
  
  /* Close Dataset */ 
  if (H5Dclose(did) == -1)
  {
    fprintf(stderr,"Cannot close dataset for %s.\n", MetadataName);
    exit(1);
  }
  return (0);
}

/*
Temp read HDF5 metadata.
*/
int TempRead(const char *MetadataName)
{
  char *MetaBuffer;
  hid_t did;
  hid_t dtype;
  hid_t dspace;
  size_t TotalSize;

  /* Open dataset */
  if ((did=H5Dopen(gid, MetadataName)) < 0) 
  {
    fprintf(stderr,"Cannot open dataset %s.\n", MetadataName);
    exit(1);
  }

  /* Get dataset type id */
  if ((dtype=H5Dget_type(did)) < 0) 
  {
    fprintf(stderr,"Cannot get dataset type %s.\n", MetadataName);
    exit(1);
  }  

  /* Get dataspace id */ 
  if ((dspace=H5Dget_space(did)) < 0) 
  {
    fprintf(stderr,"Cannot get dataset space %s.\n", MetadataName);
    exit(1);
  }   
 
  TotalSize=H5Tget_size(dtype);
  MetaBuffer=(char *)calloc(TotalSize+100, 1);

  /* Read dataset */ 
  if (H5Dread(did, dtype, dspace, H5S_ALL,H5P_DEFAULT, MetaBuffer) < 0 ) 
  {
    fprintf(stderr,"Cannot read dataset %s.\n", MetadataName);
    exit(1);
  }

  fprintf(MetadataInHDF, "%s", MetaBuffer);

  /* Close dataspace */ 
  if (H5Sclose(dspace) == -1)
  {
    fprintf(stderr,"Cannot close dataspace for %s.\n", MetadataName);
    exit(1);
  }
  
  /* Close datatype */ 
  if (H5Tclose(dtype) == -1)
  {
    fprintf(stderr,"Cannot close type for %s.\n", MetadataName);
    exit(1);
  }
  
  /* Close Dataset */ 
  if (H5Dclose(did) == -1)
  {
    fprintf(stderr,"Cannot close dataset for %s.\n", MetadataName);
    exit(1);
  }
  return (0);
}

/* Write HDF5 metadata. */ 
int MetaWrite(hid_t fid, char write, char update)
{

  /* Open group. */
  if ((gid=H5Gopen(fid, "HDFEOS INFORMATION"))==-1) 
  {
    fprintf(stderr,"Cannot open metadata group HDFEOS INFORMATION.\n");
    exit(1);
  }

  /*
  Unlink old metadata.
  */
  if ((H5Giterate(fid, "/HDFEOS INFORMATION", NULL, Unlink, NULL))<0)
  {
    fprintf(stderr, "Cannot iterate HDFEOS5 file.\n");
    exit(1);
  }

  fprintf(stderr,"It is writing metadata ...\n");

  /* 
  Read coremetadata, archivemetadata and collectionmetadata 
  by find the groups. If more than 65535, separate it.
  */

  /* Replace. */
  if (write=='y')
  {
    WriteGroup(MetadataFile, "collectionmetadata", "COLLECTIONMETADATA");
    WriteGroup(MetadataFile, "coremetadata", "INVENTORYMETADATA");
    WriteGroup(MetadataFile, "archivemetadata", "ARCHIVEMETADATA");
  }
  /* Merge. */
  if (update=='y')
  {
    WriteGroup(MetadataFile, "collectionmetadata", "COLLECTIONMETADATA");
    WriteGroup(MetadataFile, "coremetadata", "INVENTORYMETADATA");
    WriteGroup(TempFile, "archivemetadata", "ARCHIVEMETADATA");
  }

  /*  Close group. */
  if (H5Gclose(gid) == -1)
  {
    fprintf(stderr,"Cannot close group for HDF-EOS file\n");
    exit(1);
  }

  return 0;
}

/* Write MetadataFile to hdfFile. */
int WriteGroup(FILE *MFile, char *MetadataName, char *GroupName)
{
  int i=0, strsize;
  size_t TotalSize=0;
  char *MetaBuffer, *MetaPoint;
  char oLine[1024], aLine[1024], tempLine[1024], outLine[1024];
  char MetadataNamen[256], startGroup[256], endGroup[256];

  strcpy(startGroup, "GROUP=");
  strcat(startGroup, GroupName);
  strcpy(endGroup, "END_GROUP=");
  strcat(endGroup, GroupName);

  /*
  Maximum length of a vdata record: 65535  <- from the User's Guide
  Maximum order size:  65535    <-The rest of these from ./hdf/src/hlimits.h
  MAX_FIELD_SIZE: 65535
  Max name length: 65
  Max NO. of fields: 256
  MAX Field Name length: 128
  */

  MetaBuffer=(char *)malloc(65535);
  MetaPoint=MetaBuffer;

  fseek(MFile, 0, 0);

  while (fgets(oLine, 1024, MFile))
  {
    LtoU(oLine, aLine);
    strcpy(tempLine, aLine);
    noSpaceLine(tempLine, outLine);
    if (strstr(outLine, startGroup)!=NULL)
    {
      strsize=strlen(oLine);
      TotalSize+=strsize;
      memcpy(MetaPoint, oLine, strsize);
      MetaPoint+=strsize;
      while (fgets(oLine, 1024, MFile))
      {
        strsize=strlen(oLine);
        TotalSize+=strsize;
        /* Need to add "END\n", so 65535-4=65531 */
        if (TotalSize>65531)
        {
          TotalSize-=strsize;
          sprintf(MetadataNamen, "%s.%d", MetadataName, i);
          WriteDataset(MetadataNamen, MetaBuffer, TotalSize); 
          free(MetaBuffer);
          i++;
          TotalSize=strsize;
          MetaBuffer=(char *)malloc(65535);
          MetaPoint=MetaBuffer;
        }
        memcpy(MetaPoint, oLine, strsize);
        MetaPoint+=strsize;
        LtoU(oLine, aLine);
        strcpy(tempLine, aLine);
        noSpaceLine(tempLine, outLine);
        if (strstr(outLine, endGroup)!=NULL) break;
      }
      memcpy(MetaPoint, "END\n", 4);
      MetaPoint+=4;
      TotalSize+=4;
      sprintf(MetadataNamen, "%s.%d", MetadataName, i);
      WriteDataset(MetadataNamen, MetaBuffer, TotalSize); 
      free(MetaBuffer);
    }
  }

  return (0);
}

/* Write dataset. */
int WriteDataset(char *MetadataNamen, char *MetaBuffer, size_t TotalSize) 
{
  hid_t did, dtype, dspace;
  hsize_t dims[1];

  /*  Create new metadata. */
  dtype = H5Tcopy (H5T_C_S1);
  if ((H5Tset_size (dtype, TotalSize))<0)
  {
    fprintf(stderr, "Cannot set size.\n");
    exit(1);
  }
  dims[0] = 1;
  dspace = H5Screate_simple(1, dims, NULL);
  if ((did = H5Dcreate(gid, MetadataNamen, dtype, dspace, H5P_DEFAULT))<0)
  {
    fprintf(stderr, "Cannot create metadata.\n");
    exit(1);
  }

  /*  Write new metadata. */
  fprintf(stderr,"Writing metadata: %s.\n", MetadataNamen);
  if (H5Dwrite(did, dtype, dspace, H5S_ALL, H5P_DEFAULT, MetaBuffer) < 0 ) 
  {
    fprintf(stderr,"Cannot write to dataset %s.\n", MetadataNamen);
    exit(1);
  }

  /*  Close dataspace. */
  if (H5Sclose(dspace) == -1)
  {
    fprintf(stderr,"Cannot close dataspace for %s.\n", MetadataNamen);
    exit(1);
  }

  /*  Close datatype. */
  if (H5Tclose(dtype) == -1)
  {
    fprintf(stderr,"Cannot close datatype for %s.\n", MetadataNamen);
    exit(1);
  }

  /*  Close Dataset.  */
  if (H5Dclose(did) == -1)
  {
    fprintf(stderr,"Cannot close dataset for %s.\n", MetadataNamen);
    exit(1);
  }

  return (0);
}

/* Get value of ODL attribute. */
int GetValue(FILE *metadataFile, char *GroupName,  char *attributeName, char *valueName, char *returnValue)
{
  AGGREGATE root, aAggregate, bAggregate;
  PARAMETER aParameter;
  VALUE aValue;
  int  status;

  fseek(metadataFile, 0, 0);
  root=NewAggregate(NULL, KA_GROUP, "test", NULL);

  /* Read odl, no error return 1 and error return other. */
  /* fprintf(stderr,"Reading and checking odl.\n"); */
  status=ReadLabel(metadataFile, root);
  if (status!=1)
  {
    fprintf(stderr, "There are syntax errors in metadata file.\n");
    exit(1);
  }
  bAggregate=FindGroup(root, GroupName);
  if (bAggregate==NULL)
  {
    /* fprintf(stderr, "No %s group.\n", GroupName); */
    /* exit(1); */
    sprintf(returnValue, "No group");
    return (0);
  }
  aAggregate=FindAggregate(bAggregate, attributeName);
  if (aAggregate==NULL)
  {
    /* fprintf(stderr, "No %s in metadata file.\n", attributeName); */
    /* exit(1); */
    sprintf(returnValue, "No attribute");
    return (0);
  }
  /* PrintLabel(aAggregate); */

  /* Find parameter. */
  aParameter=FindParameter(aAggregate, valueName);
  if (aParameter==NULL)
  {
    /* fprintf(stderr, "No %s parameter in metadata file.\n", attributeName); */
    /* exit(1); */
    sprintf(returnValue, "No parameter");
    return (0);
  }

  /* Check value. */
  aValue=FirstValue(aParameter);
  if (aValue==NULL)
  {
    /* fprintf(stderr, "No %s value in metadata file.\n", attributeName); */
    /* exit(1); */
    sprintf(returnValue, "No value");
    return (0);
  }

  if (aValue->item.type==TV_STRING)
  {
    /* print string and time. */
    sprintf(returnValue, "%s", aValue->item.value.string);
  }
  if (aValue->item.type==TV_INTEGER)
  {
    /* print integer. */
    sprintf(returnValue, "%i", aValue->item.value.integer.number);
  }

  RemoveAggregate(root);

  return (0);
}

/* Print help message. */
void PrintHelp(void)
{
  fprintf(stderr,"\nUsage: \n");
  fprintf(stderr,"he5mu HDFEOS5FileName -xmf [MetadataFileName]\n\n");
  fprintf(stderr,"   HDFEOS5FileName--HDFEOS5 file. Required.\n");
  fprintf(stderr,"   -x Extract metadata from HDFEOS5 file.\n");
  fprintf(stderr,"   -m Update metadata (merge current metadata with input). \n");
  fprintf(stderr,"   -f Forcibly replace metadata with metadata from input.\n");
  fprintf(stderr,"   MetadataFileName--Metadata file. Optional (default is standard input/output).\n");
  fprintf(stderr,"   -h Print help message.\n");
  fprintf(stderr,"\n");
  exit(1);
}

/* Convert low case to upcase. */ 
void LtoU(char *inLine, char *outLine)
{
  unsigned int i;
  char astring, bstring;
  unsigned int ilen = strlen(inLine);

  for (i=0; i<ilen; i++)
  {
    astring=inLine[i];
    bstring=toupper(astring);
    outLine[i]=bstring;
  }
  outLine[ilen]='\0';
}

/* Change line with space to no space line. */
void noSpaceLine(char *inLine, char *outLine)
{
  char *astring, *bstring;

  /* Get no space line. */
  strcpy(outLine, "");
  astring=strtok(inLine, "\n");
  bstring=strtok(astring, "	");
  astring=strtok(bstring, " ");
  while(astring)
  {
    strcat(outLine, astring);
    astring=strtok(NULL, " ");
  }
}

/* Check INVENTORYMETADATA, COREMETADATA, COLLECTIONMETADATA groups */
int MetaGroup(FILE *metadataFile,  char *GroupName)
{
  AGGREGATE root, aAggregate;
  int  status;

  fseek(metadataFile, 0, 0);
  root=NewAggregate(NULL, KA_GROUP, "test", NULL);

  status=ReadLabel(metadataFile, root);
  if (status!=1)
  {
    fprintf(stderr, "There are syntax errors in metadata file.\n");
    exit(1);
  }
  aAggregate=FindGroup(root, GroupName);
  if (aAggregate==NULL)
  {
     return 0;
  }
  else
  {
     return 1;
  }
}

void CheckValue(FILE *TempFile, FILE *MetadataFile, char *GroupName)
{
  char HDFValue[256], METValue[256], attributeName[256], valueName[256];
  char ParameterName[3][20]={"SHORTNAME", "PRODUCTIONDATETIME", "VERSIONID"};
  int i;

  for (i=0; i<3; i++)
  {
    fseek(TempFile, 0, 0);
    fseek(MetadataFile, 0, 0);
    strcpy(attributeName, ParameterName[i]);
    strcpy(valueName, "VALUE");
    GetValue(MetadataFile, GroupName, attributeName, valueName, METValue);
    GetValue(TempFile, GroupName, attributeName, valueName, HDFValue);
    if (strcmp(HDFValue, METValue)!=0)
    {  
      fprintf(stderr,"%s values: %s and %s in metadata file and HDFEOS5 file don't match.\n",
	      attributeName, METValue, HDFValue);
      exit(1);
    }
  }
}

